/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTActor.h>
#include <RTActorClass.h>
#include <RTActorRef.h>
#include <RTActor_class.h>
#include <RTAsciiEncoding.h>
#include <RTComponentDescriptor.h>
#include <RTDebugger.h>
#include <RTDiag.h>
#include <RTDiagBuffer.h>
#include <RTDiagStream.h>
#include <RTFieldDescriptor.h>
#include <RTPortDescriptor.h>
#include <RTProtocol.h>
#include <RTRelayDescriptor.h>
#include <RTRelayPort.h>

static const char RTunknown[] = "?";

static void show_object( RTDiagStream & out,
			 int            id,
			 const char   * name,
			 int            size = 1 )
{
    out << "  " << id << ": " << name;

    if( size != 1 )
	out << "[" << size << "]";
}

static int show_attributes( RTDiagStream        & out,
			    const RTActor       * actor,
			    const RTActor_class * info )
{
    int shown = ( info->super != (const RTActor_class *)0 )
		  ? show_attributes( out, actor, info->super )
		  : 0;

    int                       count     = info->numAttributes;
    const RTFieldDescriptor * attribute = info->attributes;

    for( ; --count >= 0; ++attribute, ++shown )
    {
	if( shown == 0 )
	    out << "\nAttributes:\n";
	out << "  ";
	show_object( out, shown, attribute->name );

#if OBJECT_ENCODE
	out << " == ";

	if( attribute->type == (const RTObject_class *)0 )
	    out << "Unknown type";
	else
	{
	    RTDiagBuffer    buffer( &out );
	    RTAsciiEncoding ascii( &buffer );

	    if( attribute->encode( actor, &ascii ) == 0 || ascii.flush() == 0 )
		out << " (encoding failure)";
	}
#endif

	out << "\n";
    }

    return shown;
}

void RTDebugger::printActorInfo( RTActor * actor )
{
    RTDiagStream & out       = RTDiag::out();
    const char   * typeName  = actor->getTypeName();
    const char   * stateName = actor->getCurrentStateString();
    const char   * refName   = (const char *)0;

    if( actor->getReference() != (RTActorRef *)0 )
	refName = actor->getReference()->getName();

    if( typeName  == (const char *)0 ) typeName  = RTunknown;
    if( refName   == (const char *)0 ) refName   = RTunknown;
    if( stateName == (const char *)0 ) stateName = RTunknown;

    out << "ClassName    : " << typeName
	<< "\nReferenceName: " << refName
	<< "\nCurrentState : " << stateName
	<< "\nAddress      : (" << typeName << "_Actor*)" << (const void *)actor
	<< "\n";

#if OBSERVABLE
    if( actor->probe == (RTActorProbe *)0 )
	out << "No ";

    out << "Actor Probe attached.\n";
#endif

    int                      attached = 0;
    const RTActor_class    * info     = actor->getActorData();
    const RTActorClass     * type     = info->type;
    int                      i;
    const RTPortDescriptor * port;
    int                      id;

/*
 *   Output list of relay ports
 */

    if( type->numRelays != 0 )
    {
	out << "\nRelay ports:\n";

	int                       relayId;
	const RTRelayDescriptor * relay;

	for( relayId = 0, relay = type->relays;
	     relayId < type->numRelays;
	     ++relayId,   ++relay )
	{
	    out << "  ";
	    show_object( out, relayId, relay->name, relay->replication );
	    out << "\n";
	}
    }

/*
 *   Output list of end ports
 */

    int          visibility = RTPortDescriptor::VisibilityPublic;
    const char * visName    = "Public";

    for(;;)
    {
	for( i = info->numPorts, port = info->ports, id = 0; --i >= 0; ++port )
	{
	    if( ( port->properties & RTPortDescriptor::MaskKind )
				  == RTPortDescriptor::KindSpecial )
		continue;

	    if( ( port->properties & RTPortDescriptor::MaskVisibility )
				  != visibility )
		continue;

	    if( id == 0 )
		out << "\n" << visName << " end ports:\n";

	    RTProtocol * ref = RTObjectOf( RTProtocol, port->offset, actor );

	    out << "  ";
	    show_object( out, id++, port->name, port->replication );
	    out << " (";
	    if( ( port->properties & RTPortDescriptor::MaskKind )
				  == RTPortDescriptor::KindWired )
	    {
		out << "wired";
	    }
	    else if( ref->isRegistered() != 0 )
		out << ( ( ref->getFlags() & RTProtocol::RegisteredAsSAP ) != 0
			    ? "SAP,"
			    : "SPP," )
		    << ref->getRegisteredName();
	    else
		out << "unregistered";
	    out << ")\n";
	}

	if( visibility == RTPortDescriptor::VisibilityProtected )
	    break;

	visibility = RTPortDescriptor::VisibilityProtected;
	visName    = "Protected";
    }

/*
 *   Output list of components
 */

    int nComponents = info->numComponents;

    if( nComponents != 0 )
    {
	out << "\nComponents:\n";

	for( i = 0; i < nComponents; ++i )
	{
	    RTActorRef * ref = RTObjectOf( RTActorRef,
					   info->components[i].offset, actor );
	    out << "  ";
	    show_object( out, i + 1, info->components[i].name, ref->size() );
	    out << "\n";
	}
    }

/*
 *   Output list of ESVs
 */

    (void)show_attributes( out, actor, info );

    if( attached != 0 )
	out << "\n* probe attached\n";
}
